import React, { useState, useRef, useEffect, useCallback } from 'react';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var _excluded = ["value", "onChange", "highlight"];
var styles = {
  pad: {
    display: 'inline-block',
    overflowWrap: 'break-word'
  },
  pre: {
    margin: '0px',
    whiteSpace: 'pre-wrap'
  },
  absolute: {
    position: 'absolute',
    left: 0,
    right: 0,
    top: 0,
    bottom: 0
  },
  input: {
    resize: 'none',
    display: 'block',
    color: 'var(--editor-text-color)',
    backgroundColor: 'var(--editor-background-color)'
  }
};

var inputStyle = _extends({}, styles.pad, styles.input, styles.absolute);

var codeStyle = _extends({}, styles.pad, styles.absolute);

function Editor(_ref) {
  var _ref$value = _ref.value,
      value = _ref$value === void 0 ? '' : _ref$value,
      _ref$onChange = _ref.onChange,
      onChange = _ref$onChange === void 0 ? function () {} : _ref$onChange,
      _ref$highlight = _ref.highlight,
      highlight = _ref$highlight === void 0 ? function () {} : _ref$highlight,
      props = _objectWithoutPropertiesLoose(_ref, _excluded);

  var _useState = useState(value),
      text = _useState[0],
      setText = _useState[1];

  var _useState2 = useState(highlight(text)),
      output = _useState2[0],
      setOutput = _useState2[1];

  var codeRef = useRef();
  var textareaRef = useRef();

  function update(code) {
    var highlighted = highlight(code);
    setText(code);
    setOutput(highlighted);
    onChange(code);
  }

  useEffect(function () {
    update(value);
  }, [value]);
  var scroll = useCallback(function () {
    var code = codeRef.current;
    var textarea = textareaRef.current;

    if (code && textarea) {
      code.style.transform = "translateY(" + -textarea.scrollTop + "px)";
    }
  }, []);
  useEffect(function () {
    var textarea = textareaRef.current;

    if (textarea) {
      textarea.addEventListener('scroll', scroll);
    }

    return function () {
      textarea.removeEventListener('scroll', scroll);
    };
  }, []);

  function onInput(event) {
    var code = event.target.value || '';
    update(code);
  }

  return /*#__PURE__*/React.createElement("div", props, /*#__PURE__*/React.createElement("pre", {
    style: styles.pre
  }, /*#__PURE__*/React.createElement("code", {
    style: codeStyle,
    ref: codeRef,
    dangerouslySetInnerHTML: {
      __html: output
    }
  })), /*#__PURE__*/React.createElement("textarea", {
    ref: textareaRef,
    style: inputStyle,
    value: text,
    onChange: onInput
  }));
}

export { Editor };
